import { Address, Encoder, IAccountMeta, IInstruction, IInstructionWithAccounts, IInstructionWithData, ReadonlyAccount, ReadonlySignerAccount, TransactionSigner, WritableAccount } from '@solana/web3.js';
import { TOKEN22_PROGRAM_ID } from '../../programIds';
export interface TokenMetadata {
    updateAuthority: Address | null;
    mint: Address;
    name: string;
    symbol: string;
    uri: string;
    additionalMetadata: AdditionalMetadata[];
}
export interface AdditionalMetadata {
    key: string;
    value: string;
}
export type InitializeTokenMetadataInput<TAccountMetadata extends string = string, TAccountUpdateAuthority extends string = string, TAccountMint extends string = string, TAccountMintAuthority extends string = string> = {
    metadata: Address<TAccountMetadata>;
    updateAuthority: Address<TAccountUpdateAuthority>;
    mint: Address<TAccountMint>;
    mintAuthority: TransactionSigner<TAccountMintAuthority>;
    data: InitializeTokenMetadataInstructionDataArgs['data'];
};
export interface InitializeTokenMetadataInstructionDataArgs {
    data: TokenMetadataInstructionArgs;
}
export interface TokenMetadataInstructionArgs {
    name: string;
    symbol: string;
    uri: string;
}
export type InitializeTokenMetadataInstruction<TProgram extends string = typeof TOKEN22_PROGRAM_ID, TAccountMetadata extends string | IAccountMeta<string> = string, TAccountUpdateAuthority extends string | IAccountMeta<string> = string, TAccountMint extends string | IAccountMeta<string> = string, TAccountMintAuthority extends string | IAccountMeta<string> = string, TRemainingAccounts extends readonly IAccountMeta<string>[] = []> = IInstruction<TProgram> & IInstructionWithData<Uint8Array> & IInstructionWithAccounts<[
    TAccountMetadata extends string ? WritableAccount<TAccountMetadata> : TAccountMetadata,
    TAccountUpdateAuthority extends string ? ReadonlyAccount<TAccountUpdateAuthority> : TAccountUpdateAuthority,
    TAccountMint extends string ? ReadonlyAccount<TAccountMint> : TAccountMint,
    TAccountMintAuthority extends string ? ReadonlySignerAccount<TAccountMintAuthority> : TAccountMintAuthority,
    ...TRemainingAccounts
]>;
export declare function getInitializeTokenMetadataInstruction<TAccountMetadata extends string, TAccountUpdateAuthority extends string, TAccountMint extends string, TAccountMintAuthority extends string>(input: InitializeTokenMetadataInput): InitializeTokenMetadataInstruction<typeof TOKEN22_PROGRAM_ID, TAccountMetadata, TAccountUpdateAuthority, TAccountMint, TAccountMintAuthority>;
export declare function getInitializeTokenMetadataInstructionDataEncoder(): Encoder<InitializeTokenMetadataInstructionDataArgs>;
export declare function getTokenMetadataArgsEncoder(): Encoder<TokenMetadataInstructionArgs>;
export type UpdateFieldInput<TAccountMetadata extends string = string, TAccountUpdateAuthority extends string = string> = {
    metadata: Address<TAccountMetadata>;
    updateAuthority: TransactionSigner<TAccountUpdateAuthority>;
    data: UpdateFieldInstructionDataArgs['data'];
};
export interface UpdateFieldInstructionDataArgs {
    data: UpdateFieldInstructionArgs;
}
export interface UpdateFieldInstructionArgs {
    key: string;
    value: string;
}
export type UpdateFieldInstruction<TProgram extends string = typeof TOKEN22_PROGRAM_ID, TAccountMetadata extends string | IAccountMeta<string> = string, TAccountUpdateAuthority extends string | IAccountMeta<string> = string> = IInstruction<TProgram> & IInstructionWithData<Uint8Array> & IInstructionWithAccounts<[
    TAccountMetadata extends string ? WritableAccount<TAccountMetadata> : TAccountMetadata,
    TAccountUpdateAuthority extends string ? ReadonlySignerAccount<TAccountUpdateAuthority> : TAccountUpdateAuthority
]>;
export declare function getUpdateFieldInstruction<TAccountMetadata extends string, TAccountUpdateAuthority extends string>(input: UpdateFieldInput): UpdateFieldInstruction<typeof TOKEN22_PROGRAM_ID, TAccountMetadata, TAccountUpdateAuthority>;
export declare function getUpdateFieldInstructionDataEncoder(): Encoder<UpdateFieldInstructionDataArgs>;
export declare function getUpdateFieldArgsEncoder(): Encoder<UpdateFieldInstructionArgs>;
//# sourceMappingURL=tokenMetadata.d.ts.map