import { Address, KeyPairSigner, TransactionSigner } from '@solana/web3.js';
import { Client } from '../setup';
import { TokenMetadataInstructionArgs } from './extensions/tokenMetadata';
export interface TokenArgs {
    client: Client;
    payer: TransactionSigner;
    mint: Address;
    owner: Address;
    tokenProgram?: Address;
}
export interface MintArgs {
    client: Client;
    payer: TransactionSigner;
    mintAuthority: Address;
    freezeAuthority: Address | null;
    decimals?: number;
    tokenProgram?: Address;
}
export interface MetadataMintArgs {
    client: Client;
    payer: TransactionSigner;
    mintAuthority: TransactionSigner;
    freezeAuthority: Address | null;
    decimals?: number;
    tokenProgram?: Address;
    data: TokenMetadataInstructionArgs;
}
export interface T22NftArgs {
    client: Client;
    payer: TransactionSigner;
    owner: Address;
    mintAuthority: TransactionSigner;
    freezeAuthority: Address | null;
    decimals?: number;
    tokenProgram?: Address;
    data: TokenMetadataInstructionArgs;
    royalties: Record<string, string>;
}
export declare const createMint: (args: MintArgs) => Promise<Address>;
export declare const createToken: (args: TokenArgs) => Promise<Address>;
export declare const createAta: (args: TokenArgs) => Promise<Address>;
export interface Mint {
    mint: Address;
    decimals: number;
}
export interface CreateAndMintToArgs {
    client: Client;
    mintAuthority: KeyPairSigner;
    payer?: KeyPairSigner;
    recipient?: Address;
    decimals?: number;
    initialSupply?: bigint;
    tokenProgram?: Address;
}
export declare const createAndMintTo: (args: CreateAndMintToArgs) => Promise<[Mint, Address | null]>;
export declare const createMintWithMetadataPointer: (args: Omit<MintArgs, 'tokenProgram'>) => Promise<Address>;
export declare const createMintWithTransferHook: (args: Omit<MintArgs, 'tokenProgram'>) => Promise<Address>;
export declare const createMintWithMetadata: (args: MetadataMintArgs) => Promise<Address>;
export declare const createT22Nft: (args: Omit<T22NftArgs, 'royalties'>) => Promise<[Address, Address]>;
export declare const createT22NftWithRoyalties: (args: T22NftArgs) => Promise<[Address, Address]>;
//# sourceMappingURL=token.d.ts.map